package org.sunbird.mimetype.mgr.impl

import java.io.File
import java.util

import com.google.common.io.Resources
import org.scalamock.scalatest.AsyncMockFactory
import org.scalatest.{AsyncFlatSpec, Matchers}
import org.sunbird.models.UploadParams
import org.sunbird.cloudstore.StorageService
import org.sunbird.common.dto.{Request, Response}
import org.sunbird.common.exception.ClientException
import org.sunbird.graph.{GraphService, OntologyEngineContext}
import org.sunbird.graph.dac.model.Node

import scala.concurrent.{ExecutionContext, Future}

class EcmlMimeTypeMgrImplTest extends AsyncFlatSpec with Matchers with AsyncMockFactory{

    implicit val ss = mock[StorageService]

    it should "Throw Client Exception for null file url" in {
        val exception = intercept[ClientException] {
            new EcmlMimeTypeMgrImpl().upload("do_1234", getNode(), "", None, UploadParams())
        }
        exception.getMessage shouldEqual "Please Provide Valid File Url!"
    }

    it should "Throw Client Except for non zip file " in {
        val exception = intercept[ClientException] {
            new EcmlMimeTypeMgrImpl().upload("do_1234", getNode(), new File(Resources.getResource("sample.pdf").toURI), None, UploadParams())
        }
        exception.getMessage shouldEqual "INVALID_CONTENT_PACKAGE_FILE_MIME_TYPE_ERROR | [The uploaded package is invalid]"
    }


    it should "upload ECML zip file and return public url" in {
        val node = getNode()
        val identifier = "do_1234"
        implicit val ss = mock[StorageService]
        (ss.uploadFile(_:String, _: File, _: Option[Boolean])).expects(*, *, *).returns(Array(identifier, identifier)).repeated(3)
        (ss.uploadDirectory(_:String, _:File, _: Option[Boolean])).expects(*, *, *)
        val resFuture = new EcmlMimeTypeMgrImpl().upload(identifier, node, new File(Resources.getResource("validecml.zip").toURI), None, UploadParams())
        resFuture.map(result => {
            assert(null != result)
            assert(result.nonEmpty)
            assert("do_123" == result.getOrElse("identifier",""))
        })

        assert(true)
    }

    it should "upload ECML with json zip file and return public url" in {
        val node = getNode()
        val identifier = "do_1234"
        implicit val ss = mock[StorageService]
        (ss.uploadFile(_:String, _: File, _: Option[Boolean])).expects(*, *, *).returns(Array(identifier, identifier))
        (ss.uploadDirectory(_:String, _:File, _: Option[Boolean])).expects(*, *, *)
        val resFuture = new EcmlMimeTypeMgrImpl().upload(identifier, node, new File(Resources.getResource("validecml_withjson_new.zip").toURI), None, UploadParams())
        resFuture.map(result => {
            assert(null != result)
            assert(result.nonEmpty)
            assert("do_123" == result.getOrElse("identifier",""))
        })

        assert(true)
    }

    it should "upload ECML with json zip file URL and return public url" in {
        val node = getNode()
        val identifier = "do_1234"
        implicit val ss = mock[StorageService]
        (ss.uploadFile(_:String, _: File, _: Option[Boolean])).expects(*, *, *).returns(Array(identifier, identifier)).anyNumberOfTimes()
        (ss.uploadDirectory(_:String, _:File, _: Option[Boolean])).expects(*, *, *).anyNumberOfTimes()
        val resFuture = new EcmlMimeTypeMgrImpl().upload(identifier, node, "https://sunbirddev.blob.core.windows.net/sunbird-content-dev/content/assets/do_113322230485778432142/validecml_withjson_new.zip", None, UploadParams())
        resFuture.map(result => {
            assert(null != result)
            assert(result.nonEmpty)
            assert("do_123" == result.getOrElse("identifier",""))
        })

        assert(true)
    }

    it should "review ECML having json body and return result" in {
        implicit val ss = mock[StorageService]
        implicit val oec: OntologyEngineContext = mock[OntologyEngineContext]
        val graphDB = mock[GraphService]
        val body = """{"theme":{"id":"theme","version":"1.0","startStage":"2ec26aa4-2996-4969-b948-20cd99164ed3","stage":[{"x":0,"y":0,"w":100,"h":100,"id":"2ec26aa4-2996-4969-b948-20cd99164ed3","rotate":null,"config":{"__cdata":"{\"opacity\":100,\"strokeWidth\":1,\"stroke\":\"rgba(255, 255, 255, 0)\",\"autoplay\":false,\"visible\":true,\"color\":\"#FFFFFF\",\"genieControls\":false,\"instructions\":\"\"}"},"param":[{"name":"next","value":"7cb8c4bc-20d6-49d7-9480-bd6823bdc7c5"}],"manifest":{"media":[]},"org.ekstep.text":[{"x":10,"y":20,"minWidth":20,"w":35,"maxWidth":500,"fill":"#000000","fontStyle":"normal","fontWeight":"normal","stroke":"rgba(255, 255, 255, 0)","strokeWidth":1,"opacity":1,"editable":false,"version":"V2","offsetY":0.2,"h":55.24,"rotate":0,"textType":"text","lineHeight":1,"z-index":0,"font":"'Noto Sans', 'Noto Sans Bengali', 'Noto Sans Malayalam', 'Noto Sans Gurmukhi', 'Noto Sans Devanagari', 'Noto Sans Gujarati', 'Noto Sans Telugu', 'Noto Sans Tamil', 'Noto Sans Kannada', 'Noto Sans Oriya', 'Noto Nastaliq Urdu', -apple-system, BlinkMacSystemFont, Roboto, Oxygen-Sans, Ubuntu, Cantarell, 'Helvetica Neue'","fontsize":48,"weight":"","id":"499bec78-1ba7-4375-80d9-be3831a9e3b3","config":{"__cdata":"{\"opacity\":100,\"strokeWidth\":1,\"stroke\":\"rgba(255, 255, 255, 0)\",\"autoplay\":false,\"visible\":true,\"text\":\"हिन्दी जिसके मानकीकृत रूप को मानक हिंदी कहा जाता है, विश्व की एक प्रमुख भाषा है एवं भारत की एक राजभाषा है। केन्द्रीय स्तर पर भारत में सह-आधिकारिक भाषा अंग्रेजी है। यह हिन्दुस्तानी भाषा की एक मानकीकृत रूप है जिसमें संस्कृत के तत्सम तथा तद्भव शब्दों का प्रयोग अधिक है और अरबी-फ़ारसी शब्द कम हैं। हिन्दी संवैधानिक रूप से भारत की राजभाषा और भारत की सबसे अधिक बोल…\",\"color\":\"#000000\",\"fontfamily\":\"'Noto Sans', 'Noto Sans Bengali', 'Noto Sans Malayalam', 'Noto Sans Gurmukhi', 'Noto Sans Devanagari', 'Noto Sans Gujarati', 'Noto Sans Telugu', 'Noto Sans Tamil', 'Noto Sans Kannada', 'Noto Sans Oriya', 'Noto Nastaliq Urdu', -apple-system, BlinkMacSystemFont, Roboto, Oxygen-Sans, Ubuntu, Cantarell, 'Helvetica Neue'\",\"fontsize\":18,\"fontweight\":false,\"fontstyle\":false,\"align\":\"left\"}"}}]},{"x":0,"y":0,"w":100,"h":100,"id":"7cb8c4bc-20d6-49d7-9480-bd6823bdc7c5","rotate":null,"config":{"__cdata":"{\"opacity\":100,\"strokeWidth\":1,\"stroke\":\"rgba(255, 255, 255, 0)\",\"autoplay\":false,\"visible\":true,\"color\":\"#FFFFFF\",\"genieControls\":false,\"instructions\":\"\"}"},"param":[{"name":"previous","value":"2ec26aa4-2996-4969-b948-20cd99164ed3"},{"name":"next","value":"7a8c233c-b634-4883-8c5d-eb9eaa49b39e"}],"manifest":{"media":[]},"org.ekstep.text":[{"x":10,"y":20,"minWidth":20,"w":35,"maxWidth":500,"fill":"#000000","fontStyle":"normal","fontWeight":"normal","stroke":"rgba(255, 255, 255, 0)","strokeWidth":1,"opacity":1,"editable":false,"version":"V2","offsetY":0.2,"h":55.24,"rotate":0,"textType":"text","lineHeight":1,"z-index":0,"font":"'Noto Sans', 'Noto Sans Bengali', 'Noto Sans Malayalam', 'Noto Sans Gurmukhi', 'Noto Sans Devanagari', 'Noto Sans Gujarati', 'Noto Sans Telugu', 'Noto Sans Tamil', 'Noto Sans Kannada', 'Noto Sans Oriya', 'Noto Nastaliq Urdu', -apple-system, BlinkMacSystemFont, Roboto, Oxygen-Sans, Ubuntu, Cantarell, 'Helvetica Neue'","fontsize":48,"weight":"","id":"91321a56-971c-4ea6-85da-b91ab767b4a6","config":{"__cdata":"{\"opacity\":100,\"strokeWidth\":1,\"stroke\":\"rgba(255, 255, 255, 0)\",\"autoplay\":false,\"visible\":true,\"text\":\"हिन्दी जिसके मानकीकृत रूप को मानक हिंदी कहा जाता है, विश्व की एक प्रमुख भाषा है एवं भारत की एक राजभाषा है। केन्द्रीय स्तर पर भारत में सह-आधिकारिक भाषा अंग्रेजी है। यह हिन्दुस्तानी भाषा की एक मानकीकृत रूप है जिसमें संस्कृत के तत्सम तथा तद्भव शब्दों का प्रयोग अधिक है और अरबी-फ़ारसी शब्द कम हैं। हिन्दी संवैधानिक रूप से भारत की राजभाषा और भारत की सबसे अधिक बोल…\",\"color\":\"#000000\",\"fontfamily\":\"'Noto Sans', 'Noto Sans Bengali', 'Noto Sans Malayalam', 'Noto Sans Gurmukhi', 'Noto Sans Devanagari', 'Noto Sans Gujarati', 'Noto Sans Telugu', 'Noto Sans Tamil', 'Noto Sans Kannada', 'Noto Sans Oriya', 'Noto Nastaliq Urdu', -apple-system, BlinkMacSystemFont, Roboto, Oxygen-Sans, Ubuntu, Cantarell, 'Helvetica Neue'\",\"fontsize\":18,\"fontweight\":false,\"fontstyle\":false,\"align\":\"left\"}"}}]},{"x":0,"y":0,"w":100,"h":100,"id":"7a8c233c-b634-4883-8c5d-eb9eaa49b39e","rotate":null,"config":{"__cdata":"{\"opacity\":100,\"strokeWidth\":1,\"stroke\":\"rgba(255, 255, 255, 0)\",\"autoplay\":false,\"visible\":true,\"color\":\"#FFFFFF\",\"genieControls\":false,\"instructions\":\"\"}"},"param":[{"name":"previous","value":"7cb8c4bc-20d6-49d7-9480-bd6823bdc7c5"},{"name":"next","value":"7bdb2097-8559-42a9-a828-6e486a2dba96"}],"manifest":{"media":[]},"org.ekstep.text":[{"x":10,"y":20,"minWidth":20,"w":35,"maxWidth":500,"fill":"#000000","fontStyle":"normal","fontWeight":"normal","stroke":"rgba(255, 255, 255, 0)","strokeWidth":1,"opacity":1,"editable":false,"version":"V2","offsetY":0.2,"h":55.24,"rotate":0,"textType":"text","lineHeight":1,"z-index":0,"font":"'Noto Sans', 'Noto Sans Bengali', 'Noto Sans Malayalam', 'Noto Sans Gurmukhi', 'Noto Sans Devanagari', 'Noto Sans Gujarati', 'Noto Sans Telugu', 'Noto Sans Tamil', 'Noto Sans Kannada', 'Noto Sans Oriya', 'Noto Nastaliq Urdu', -apple-system, BlinkMacSystemFont, Roboto, Oxygen-Sans, Ubuntu, Cantarell, 'Helvetica Neue'","fontsize":48,"weight":"","id":"7abc0887-0d51-444c-a070-805625a2caa4","config":{"__cdata":"{\"opacity\":100,\"strokeWidth\":1,\"stroke\":\"rgba(255, 255, 255, 0)\",\"autoplay\":false,\"visible\":true,\"text\":\"हिन्दी जिसके मानकीकृत रूप को मानक हिंदी कहा जाता है, विश्व की एक प्रमुख भाषा है एवं भारत की एक राजभाषा है। केन्द्रीय स्तर पर भारत में सह-आधिकारिक भाषा अंग्रेजी है। यह हिन्दुस्तानी भाषा की एक मानकीकृत रूप है जिसमें संस्कृत के तत्सम तथा तद्भव शब्दों का प्रयोग अधिक है और अरबी-फ़ारसी शब्द कम हैं। हिन्दी संवैधानिक रूप से भारत की राजभाषा और भारत की सबसे अधिक बोल…\",\"color\":\"#000000\",\"fontfamily\":\"'Noto Sans', 'Noto Sans Bengali', 'Noto Sans Malayalam', 'Noto Sans Gurmukhi', 'Noto Sans Devanagari', 'Noto Sans Gujarati', 'Noto Sans Telugu', 'Noto Sans Tamil', 'Noto Sans Kannada', 'Noto Sans Oriya', 'Noto Nastaliq Urdu', -apple-system, BlinkMacSystemFont, Roboto, Oxygen-Sans, Ubuntu, Cantarell, 'Helvetica Neue'\",\"fontsize\":18,\"fontweight\":false,\"fontstyle\":false,\"align\":\"left\"}"}}]},{"x":0,"y":0,"w":100,"h":100,"id":"7bdb2097-8559-42a9-a828-6e486a2dba96","rotate":null,"config":{"__cdata":"{\"opacity\":100,\"strokeWidth\":1,\"stroke\":\"rgba(255, 255, 255, 0)\",\"autoplay\":false,\"visible\":true,\"color\":\"#FFFFFF\",\"genieControls\":false,\"instructions\":\"\"}"},"param":[{"name":"previous","value":"7a8c233c-b634-4883-8c5d-eb9eaa49b39e"},{"name":"next","value":"3c47d480-99d0-46d4-97ab-2ba48ca37cc4"}],"manifest":{"media":[{"assetId":"do_1133235372209766401406"}]},"org.ekstep.video":[{"asset":"do_1133235372209766401406","y":7.9,"x":10.97,"w":78.4,"h":79.51,"rotate":0,"z-index":0,"id":"7a1e5ae0-8a08-423e-8087-621e9095b4a2","config":{"__cdata":"{\"autoplay\":true,\"controls\":true,\"muted\":false,\"visible\":true}"}}]},{"x":0,"y":0,"w":100,"h":100,"id":"3c47d480-99d0-46d4-97ab-2ba48ca37cc4","rotate":null,"config":{"__cdata":"{\"opacity\":100,\"strokeWidth\":1,\"stroke\":\"rgba(255, 255, 255, 0)\",\"autoplay\":false,\"visible\":true,\"color\":\"#FFFFFF\",\"genieControls\":false,\"instructions\":\"\"}"},"param":[{"name":"previous","value":"7bdb2097-8559-42a9-a828-6e486a2dba96"},{"name":"next","value":"503c3919-a883-4269-a648-e01cc75578eb"}],"manifest":{"media":[{"assetId":"do_1133235372209766401406"}]},"org.ekstep.video":[{"asset":"do_1133235372209766401406","y":7.9,"x":10.97,"w":78.4,"h":79.51,"rotate":0,"z-index":0,"id":"0b2eb740-bb9a-4a3b-b978-a129d2b37b3e","config":{"__cdata":"{\"autoplay\":true,\"controls\":true,\"muted\":false,\"visible\":true}"}}]},{"x":0,"y":0,"w":100,"h":100,"id":"503c3919-a883-4269-a648-e01cc75578eb","rotate":null,"config":{"__cdata":"{\"opacity\":100,\"strokeWidth\":1,\"stroke\":\"rgba(255, 255, 255, 0)\",\"autoplay\":false,\"visible\":true,\"color\":\"#FFFFFF\",\"genieControls\":false,\"instructions\":\"\"}"},"param":[{"name":"previous","value":"3c47d480-99d0-46d4-97ab-2ba48ca37cc4"},{"name":"next","value":"6ebede95-dd0b-4817-94cc-ba0181b21405"}],"manifest":{"media":[{"assetId":"do_1133235372209766401406"}]},"org.ekstep.video":[{"asset":"do_1133235372209766401406","y":7.9,"x":10.97,"w":78.4,"h":79.51,"rotate":0,"z-index":0,"id":"c62de6b6-559c-4562-8739-40c800ba1922","config":{"__cdata":"{\"autoplay\":true,\"controls\":true,\"muted\":false,\"visible\":true}"}}]},{"x":0,"y":0,"w":100,"h":100,"id":"6ebede95-dd0b-4817-94cc-ba0181b21405","rotate":null,"config":{"__cdata":"{\"opacity\":100,\"strokeWidth\":1,\"stroke\":\"rgba(255, 255, 255, 0)\",\"autoplay\":false,\"visible\":true,\"color\":\"#FFFFFF\",\"genieControls\":false,\"instructions\":\"\"}"},"param":[{"name":"previous","value":"503c3919-a883-4269-a648-e01cc75578eb"},{"name":"next","value":"summary_stage_id"}],"manifest":{"media":[{"assetId":"do_1133235374158807041407"}]},"image":[{"asset":"do_1133235374158807041407","x":29.44,"y":23.39,"w":36.32,"h":64.56,"rotate":0,"z-index":0,"id":"37dec93c-d5b4-4659-82b7-a859bc14c565","config":{"__cdata":"{\"opacity\":100,\"strokeWidth\":1,\"stroke\":\"rgba(255, 255, 255, 0)\",\"autoplay\":false,\"visible\":true}"}}]},{"x":0,"y":0,"w":100,"h":100,"rotate":null,"config":{"__cdata":"{\"opacity\":100,\"strokeWidth\":1,\"stroke\":\"rgba(255, 255, 255, 0)\",\"autoplay\":false,\"visible\":true,\"color\":\"#FFFFFF\",\"genieControls\":false,\"instructions\":\"\"}"},"id":"summary_stage_id","manifest":{"media":[{"assetId":"summaryImage"}]},"org.ekstep.summary":[{"config":{"__cdata":"{\"opacity\":100,\"strokeWidth\":1,\"stroke\":\"rgba(255, 255, 255, 0)\",\"autoplay\":false,\"visible\":true}"},"id":"summary_plugin_id","rotate":0,"x":6.69,"y":-27.9,"w":77.45,"h":125.53,"z-index":0}]}],"manifest":{"media":[{"id":"822de397-4692-4951-9c7b-31a8ad5bbb78","plugin":"org.ekstep.navigation","ver":"1.0","src":"/content-plugins/org.ekstep.navigation-1.0/renderer/controller/navigation_ctrl.js","type":"js"},{"id":"b5399218-0c7c-4858-aaa6-2ea64dae6813","plugin":"org.ekstep.navigation","ver":"1.0","src":"/content-plugins/org.ekstep.navigation-1.0/renderer/templates/navigation.html","type":"js"},{"id":"org.ekstep.navigation","plugin":"org.ekstep.navigation","ver":"1.0","src":"/content-plugins/org.ekstep.navigation-1.0/renderer/plugin.js","type":"plugin"},{"id":"org.ekstep.navigation_manifest","plugin":"org.ekstep.navigation","ver":"1.0","src":"/content-plugins/org.ekstep.navigation-1.0/manifest.json","type":"json"},{"id":"org.ekstep.text","plugin":"org.ekstep.text","ver":"1.2","src":"/content-plugins/org.ekstep.text-1.2/renderer/supertextplugin.js","type":"plugin"},{"id":"org.ekstep.text_manifest","plugin":"org.ekstep.text","ver":"1.2","src":"/content-plugins/org.ekstep.text-1.2/manifest.json","type":"json"},{"id":"5676764d-6c2b-4a62-9e19-09953443d381","plugin":"org.ekstep.video","ver":"1.5","src":"/content-plugins/org.ekstep.video-1.5/renderer/libs/video.js","type":"js"},{"id":"5153cac8-bcd1-4008-902c-a5d0324b4715","plugin":"org.ekstep.video","ver":"1.5","src":"/content-plugins/org.ekstep.video-1.5/renderer/libs/videoyoutube.js","type":"js"},{"id":"770cdc30-9350-4c70-ad6c-9258623748d5","plugin":"org.ekstep.video","ver":"1.5","src":"/content-plugins/org.ekstep.video-1.5/renderer/libs/videojs.css","type":"css"},{"id":"org.ekstep.video","plugin":"org.ekstep.video","ver":"1.5","src":"/content-plugins/org.ekstep.video-1.5/renderer/videoplugin.js","type":"plugin"},{"id":"org.ekstep.video_manifest","plugin":"org.ekstep.video","ver":"1.5","src":"/content-plugins/org.ekstep.video-1.5/manifest.json","type":"json"},{"id":"org.ekstep.summary_template_js","plugin":"org.ekstep.summary","src":"/content-plugins/org.ekstep.summary-1.0/renderer/summary-template.js","type":"js","ver":"1.0"},{"id":"org.ekstep.summary_template_css","plugin":"org.ekstep.summary","src":"/content-plugins/org.ekstep.summary-1.0/renderer/style.css","type":"css","ver":"1.0"},{"id":"org.ekstep.summary","plugin":"org.ekstep.summary","src":"/content-plugins/org.ekstep.summary-1.0/renderer/plugin.js","type":"plugin","ver":"1.0"},{"id":"org.ekstep.summary_manifest","plugin":"org.ekstep.summary","src":"/content-plugins/org.ekstep.summary-1.0/manifest.json","type":"json","ver":"1.0"},{"assetId":"summaryImage","id":"org.ekstep.summary_summaryImage","preload":true,"src":"/content-plugins/org.ekstep.summary-1.0/assets/summary-icon.jpg","type":"image"},{"id":"do_1133235372209766401406","src":"https://www.youtube.com/watch?v=Jxyx31JUEaE","assetId":"do_1133235372209766401406","type":"youtube"},{"name":"Image","id":"do_1133235374158807041407","src":"/assets/public/content/do_1133235374158807041407/artifact/do_1133235374158807041407_1626408376105_b1d586453b4e0b1535bf55332347934e.jpg","type":"image"}]},"plugin-manifest":{"plugin":[{"id":"org.ekstep.navigation","ver":"1.0","type":"plugin","depends":""},{"id":"org.ekstep.text","ver":"1.2","type":"plugin","depends":""},{"id":"org.ekstep.video","ver":"1.5","type":"widget","depends":""},{"depends":"","id":"org.ekstep.summary","type":"plugin","ver":"1.0"}]},"compatibilityVersion":4}}"""
        val response = new Response
        response.put("body", body)
        (oec.graphService _).expects().returns(graphDB).anyNumberOfTimes()
        (graphDB.readExternalProps(_: Request, _: List[String])).expects(*, *).returns(Future(response)).anyNumberOfTimes()
        val node = getNode()
        node.setIdentifier("do_2133122972131737601714")
        node.setObjectType("Content")
        node.getMetadata.put("artifactUrl", "https://sunbirddev.blob.core.windows.net/sunbird-content-dev/content/do_113105564164997120111/artifact/1599796728064_do_11310551225702809612421.zip")
        val identifier = "do_2133122972131737601714"
        val resFuture = new EcmlMimeTypeMgrImpl().review(identifier, node)
        resFuture.map(result => {
            assert(null != result)
            assert(result.nonEmpty)
            assert("Review" == result.getOrElse("status",""))
        })
    }

    it should "review ECML having xml body and return result" in {
        implicit val ss = mock[StorageService]
        implicit val oec: OntologyEngineContext = mock[OntologyEngineContext]
        val graphDB = mock[GraphService]
        val body = """<theme startStage="d6442041-7f89-8ef3-890f-443847aa4768" compatibilityVersion="2" version="1.0" id="theme"><manifest><media plugin="org.ekstep.navigation" id="org.ekstep.navigation_js" ver="1.0" src="content-plugins/org.ekstep.navigation-1.0/renderer/controller/navigation_ctrl.js" type="js"></media><media plugin="org.ekstep.navigation" id="org.ekstep.navigation_html" ver="1.0" src="content-plugins/org.ekstep.navigation-1.0/renderer/templates/navigation.html" type="js"></media><media plugin="org.ekstep.navigation" id="org.ekstep.navigation" ver="1.0" src="content-plugins/org.ekstep.navigation-1.0/renderer/plugin.js" type="plugin"></media><media plugin="org.ekstep.navigation" id="org.ekstep.navigation_manifest" ver="1.0" src="content-plugins/org.ekstep.navigation-1.0/manifest.json" type="json"></media><media plugin="org.ekstep.iterator" id="org.ekstep.iterator" ver="1.0" src="content-plugins/org.ekstep.iterator-1.0/renderer/plugin.js" type="plugin"></media><media plugin="org.ekstep.iterator" id="org.ekstep.iterator_manifest" ver="1.0" src="content-plugins/org.ekstep.iterator-1.0/manifest.json" type="json"></media><media plugin="org.ekstep.questionset" id="org.ekstep.questionset_telemetry_logger_js" ver="1.0" src="content-plugins/org.ekstep.questionset-1.0/renderer/utils/telemetry_logger.js" type="js"></media><media plugin="org.ekstep.questionset" id="org.ekstep.questionset_audio_plugin_js" ver="1.0" src="content-plugins/org.ekstep.questionset-1.0/renderer/utils/html_audio_plugin.js" type="js"></media><media plugin="org.ekstep.questionset" id="org.ekstep.questionset_feedback_popup_js" ver="1.0" src="content-plugins/org.ekstep.questionset-1.0/renderer/utils/qs_feedback_popup.js" type="js"></media><media plugin="org.ekstep.questionset" id="org.ekstep.questionset" ver="1.0" src="content-plugins/org.ekstep.questionset-1.0/renderer/plugin.js" type="plugin"></media><media plugin="org.ekstep.questionset" id="org.ekstep.questionset_manifest" ver="1.0" src="content-plugins/org.ekstep.questionset-1.0/manifest.json" type="json"></media><media plugin="org.ekstep.questionunit" id="org.ekstep.questionunit.renderer.audioicon" ver="1.1" src="content-plugins/org.ekstep.questionunit-1.1/renderer/assets/audio-icon.png" type="image"></media><media plugin="org.ekstep.questionunit" id="org.ekstep.questionunit.renderer.downarrow" ver="1.1" src="content-plugins/org.ekstep.questionunit-1.1/renderer/assets/down_arrow.png" type="image"></media><media plugin="org.ekstep.questionunit" id="org.ekstep.questionunit_js" ver="1.1" src="content-plugins/org.ekstep.questionunit-1.1/renderer/components/js/components.js" type="js"></media><media plugin="org.ekstep.questionunit" id="org.ekstep.questionunit_css" ver="1.1" src="content-plugins/org.ekstep.questionunit-1.1/renderer/components/css/components.css" type="css"></media><media plugin="org.ekstep.questionunit" id="org.ekstep.questionunit" ver="1.1" src="content-plugins/org.ekstep.questionunit-1.1/renderer/plugin.js" type="plugin"></media><media plugin="org.ekstep.questionunit" id="org.ekstep.questionunit_manifest" ver="1.1" src="content-plugins/org.ekstep.questionunit-1.1/manifest.json" type="json"></media><media plugin="org.sunbird.questionunit.quml" id="org.sunbird.questionunit.quml_manifest" ver="1.1" src="content-plugins/org.sunbird.questionunit.quml-1.1/manifest.json" type="json"></media><media plugin="org.sunbird.questionunit.quml" id="org.sunbird.questionunit.quml.plugin_js" ver="1.1" src="content-plugins/org.sunbird.questionunit.quml-1.1/renderer/plugin.js" type="plugin"></media><media plugin="org.sunbird.questionunit.quml" id="org.sunbird.questionunit.quml.feedback_popup" ver="1.1" src="content-plugins/org.sunbird.questionunit.quml-1.1/renderer/utils/quml_feedback_popup.js" type="js"></media><media plugin="org.sunbird.questionunit.quml" id="org.sunbird.questionunit.quml.feedback_close" ver="1.1" src="content-plugins/org.sunbird.questionunit.quml-1.1/renderer/assets/feedback-close.svg" type="image"></media><media plugin="org.sunbird.questionunit.quml" id="org.sunbird.questionunit.quml.play" ver="1.1" src="content-plugins/org.sunbird.questionunit.quml-1.1/renderer/assets/player-play-button.png" type="image"></media><media plugin="org.sunbird.questionunit.quml" id="org.sunbird.questionunit.quml_css" ver="1.1" src="content-plugins/org.sunbird.questionunit.quml-1.1/renderer/styles/style.css" type="css"></media><media plugin="org.ekstep.questionunit" id="4b2b66a2-9c5b-42d5-891d-ec0c3d1d516c" ver="1.1" src="/content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/katex.min.js" type="js"></media><media plugin="org.ekstep.questionunit" id="7e02c56a-c048-444d-bb2a-d4b854723adc" ver="1.1" src="content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/katex.min.css" type="css"></media><media plugin="org.ekstep.questionunit" id="04ad980a-88ac-4c55-a597-266e42240670" ver="1.1" src="content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/fonts/katex_main-bold.ttf" type="js"></media><media plugin="org.ekstep.questionunit" id="75ab3bcd-2c52-47b0-8b46-b635256ce06a" ver="1.1" src="content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/fonts/katex_main-bolditalic.ttf" type="js"></media><media plugin="org.ekstep.questionunit" id="098cc69e-8658-483e-8597-8c392a1b3545" ver="1.1" src="content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/fonts/katex_main-italic.ttf" type="js"></media><media plugin="org.ekstep.questionunit" id="102d1165-9544-480b-9ec1-1cb83937e650" ver="1.1" src="content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/fonts/katex_main-regular.ttf" type="js"></media><media plugin="org.ekstep.questionunit" id="3bc5dce9-51ca-4a59-a0d9-d87d47c44670" ver="1.1" src="content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/fonts/katex_math-bolditalic.ttf" type="js"></media><media plugin="org.ekstep.questionunit" id="a95bb693-d688-4e7a-a9bc-08e0cb6d1d62" ver="1.1" src="content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/fonts/katex_math-italic.ttf" type="js"></media><media plugin="org.ekstep.questionunit" id="15757105-9023-4773-ba2a-8e619a0dc3a6" ver="1.1" src="content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/fonts/katex_math-regular.ttf" type="js"></media><media plugin="org.ekstep.questionunit" id="5b1a9e58-8b7e-4295-a90d-593dc831262a" ver="1.1" src="content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/fonts/katex_size1-regular.ttf" type="js"></media><media plugin="org.ekstep.questionunit" id="bc6ee547-9db0-480e-b298-b5509d43207a" ver="1.1" src="content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/fonts/katex_size2-regular.ttf" type="js"></media><media plugin="org.ekstep.questionunit" id="35b0245e-d7d3-4ac0-9cac-6c2c328c8948" ver="1.1" src="content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/fonts/katex_size3-regular.ttf" type="js"></media><media plugin="org.ekstep.questionunit" id="80a19118-48eb-4859-9b0c-576a90a01148" ver="1.1" src="content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/fonts/katex_size4-regular.ttf" type="js"></media></manifest><stage x="0" y="0" id="d6442041-7f89-8ef3-890f-443847aa4768" h="100" w="100"><config><![CDATA[{'opacity':100,'strokeWidth':1,'stroke':'rgba(255, 255, 255, 0)','autoplay':false,'visible':true,'color':'#FFFFFF','genieControls':false,'instructions':''}]]></config><org.ekstep.questionset z-index="0" x="9" y="6" rotate="0" id="39306de6-b9c0-0f23-dca3-893ce0ecbf0c" h="85" w="80"><data><![CDATA[[{"identifier":"39306de6-b9c0-0f23-dca3-893ce0ecbf0c"}]]]></data><config><![CDATA[{"max_score":1,"allow_skip":true,"show_feedback":false,"shuffle_questions":false,"shuffle_options":false,"total_items":1}]]></config><org.ekstep.question x="9" y="6" pluginId="org.sunbird.questionunit.quml" id="b077184c-a43c-e0fc-1f7f-5d6806dab050" templateId="qumltemplate" h="85" type="quml" w="80" pluginVer="1.1"><data><![CDATA[{"question":"<p>A</p>","media":[],"answer":"<p>B</p>","options":[],"questionCount":0}]]></data><config><![CDATA[{"max_score":1,"partial_scoring":false,"isShuffleOption":false,"responseDeclaration":{},"metadata":{"copyright":"2020","itemType":"UNIT","code":"fd2dbbda-1876-736e-ed82-3ec2e589d3bc","subject":"Science","qlevel":"MEDIUM","qumlVersion":1,"channel":"012983850117177344161","responseDeclaration":{"responseValue":{"cardinality":"single","type":"string","correct_response":{"value":"<p>B</p>"}}},"language":["English"],"medium":"English","type":"reference","templateId":"NA","createdOn":"2020-09-11T03:12:59.107+0000","gradeLevel":["Class 10"],"appId":"dev.sunbird.portal","contentPolicyCheck":true,"lastUpdatedOn":"2020-09-11T03:13:46.143+0000","identifier":"do_1131055122768445441310","creator":"devreviewer7@yopmail.com","lastStatusChangedOn":"2020-09-11T03:12:59.107+0000","author":"devreviewer7@yopmail.com","consumerId":"028d6fb1-2d6f-4331-86aa-f7cf491a41e0","version":3,"versionKey":"1599794026143","license":"CC BY 4.0","framework":"ekstep_ncert_k-12","rejectComment":"","name":"vsa_ekstep_ncert_k-12","template_id":"NA","category":"VSA","board":"CBSE","programId":"29a5a630-c29c-11ea-b3d3-3bcdd8c1d450","status":"Draft"}}]]></config></org.ekstep.question></org.ekstep.questionset></stage><plugin-manifest><plugin depends="" id="org.ekstep.navigation" ver="1.0" type="plugin"></plugin><plugin depends="" id="org.ekstep.questionunit" ver="1.1" type="plugin"></plugin><plugin depends="" id="org.ekstep.iterator" ver="1.0" type="plugin"></plugin><plugin depends="org.ekstep.questionset.quiz,org.ekstep.iterator" id="org.ekstep.questionset" ver="1.0" type="plugin"></plugin><plugin depends="org.ekstep.questionunit" id="org.sunbird.questionunit.quml" ver="1.1" type="plugin"></plugin></plugin-manifest></theme>"""
        val response = new Response
        response.put("body", body)
        (oec.graphService _).expects().returns(graphDB).anyNumberOfTimes()
        (graphDB.readExternalProps(_: Request, _: List[String])).expects(*, *).returns(Future(response)).anyNumberOfTimes()
        val node = getNode()
        node.setIdentifier("do_2133122972131737601714")
        node.setObjectType("Content")
        node.getMetadata.put("artifactUrl", "https://sunbirddev.blob.core.windows.net/sunbird-content-dev/content/do_113105564164997120111/artifact/1599796728064_do_11310551225702809612421.zip")
        val identifier = "do_2133122972131737601714"
        val resFuture = new EcmlMimeTypeMgrImpl().review(identifier, node)
        resFuture.map(result => {
            assert(null != result)
            assert(result.nonEmpty)
            assert("Review" == result.getOrElse("status",""))
        })
    }

    def getNode(): Node = {
        val node = new Node()
        node.setIdentifier("do_1234")
        node.setMetadata(new util.HashMap[String, AnyRef](){{
            put("identifier", "do_1234")
            put("mimeType", "application/vnd.ekstep.ecml-archive")
            put("status","Draft")
            put("contentType", "Resource")
        }})
        node
    }
}
